@@ -1,14 +1,15 @@
-# Copyright 2021 VMware, Inc.
+# Copyright 2021-2023 VMware, Inc.
 # SPDX-License-Identifier: Apache-2.0
+import asyncio
 import logging
 import os
 import tempfile
 
-from minikerberos.common import KerberosCredential
-from minikerberos.communication import KerberosSocket
-from minikerberos.communication import KerbrosComm
+from minikerberos.common.creds import KerberosCredential
+from minikerberos.common.target import KerberosTarget
 from vdk.internal.core import errors
 from vdk.plugin.kerberos.base_authenticator import BaseAuthenticator
+from vdk.plugin.kerberos.vdk_kerberos_client import VdkAioKerberosClient
 
 log = logging.getLogger(__name__)
 
@@ -68,16 +69,35 @@ class MinikerberosGSSAPIAuthenticator(BaseAuthenticator):
             krb_credentials = KerberosCredential.from_keytab(
                 self._keytab_pathname, self._kerberos_principal, self._kerberos_realm
             )
-            krb_socket = KerberosSocket(self._kerberos_kdc_hostname)
-            krb_comm = KerbrosComm(krb_credentials, krb_socket)
-            krb_comm.get_TGT()
-            krb_comm.ccache.to_file(self._ccache_file)
+            krb_target = KerberosTarget(self._kerberos_kdc_hostname)
+            krb_client = VdkAioKerberosClient(krb_credentials, krb_target)
+
+            async def get_tgt():
+                await krb_client.get_TGT()
+
+            # Create a separate event loop and run minikerberos coroutines in it
+            # to avoid interfering with other plugins or with data jobs that
+            # rely on asyncio functionality, as well. The steps of the process
+            # are as follows:
+            # 1) Create a new event loop which is different from the default one
+            # 2) Execute the `get_tgt()` coroutine and wait until it is
+            #    finished.
+            # 3) After the kerberos TGT is retrieved and the coroutine has
+            #    finished, close the event loop. As it is a single coroutine,
+            #    we don't really need to do anything else.
+            loop = asyncio.new_event_loop()
+            try:
+                loop.run_until_complete(get_tgt())
+            finally:
+                loop.close()
+
+            krb_client.ccache.to_file(self._ccache_file)
             log.info(
                 f"Got Kerberos TGT for {self._kerberos_principal}@{self._kerberos_realm} "
                 f"and stored to file: {self._ccache_file}"
             )
         except Exception as e:
-            errors.log_and_throw(
+            errors.log_and_rethrow(
                 to_be_fixed_by=errors.ResolvableBy.CONFIG_ERROR,
                 log=log,
                 what_happened="Could not retrieve Kerberos TGT",
@@ -85,4 +105,6 @@ class MinikerberosGSSAPIAuthenticator(BaseAuthenticator):
                 consequences="Kerberos authentication will fail, and as a result the current process will fail.",
                 countermeasures="See stdout for details and fix the code, so that getting the TGT succeeds. "
                 "If you have custom Kerberos settings, set through environment variables make sure they are correct.",
+                exception=e,
+                wrap_in_vdk_error=True,
             )
